/*
 ************************************************************************************************
 *                                                                                              *
 *  OSA cooperative RTOS for Microchip PIC-controllers: PIC10/12/16/18/24/dsPIC                 *
 *                                                                                              *
 *  OSA is distributed under BSD license (see license.txt)                                      *
 *                                                                                              *
 *  URL:            http://wiki.pic24.ru/doku.php/en/osa/ref/intro                              *
 *                  http://picosa.narod.ru                                                      *
 *                                                                                              *
 *----------------------------------------------------------------------------------------------*
 *                                                                                              *
 *  File:           osa_mikroc16.c                                                              *
 *                                                                                              *
 *  Compilers:      mikroC PRO 3.0 and higher                                                   *
 *                                                                                              *
 *  Programmer:     Timofeev Victor                                                             *
 *                  osa@pic24.ru, testerplus@mail.ru                                            *
 *                                                                                              *
 *  Description:    mikroC PRO specific functions overloads for PIC16                           *
 *                                                                                              *
 *  History:        22.09.2009                                                                  *
 *                  03.11.2009 -    4. Increased speed of OS_CheckEvent for non-priority mode   *
 *                                                                                              *
 ************************************************************************************************
 */



volatile unsigned char    _fsr      absolute 0x04;
volatile unsigned char    _status   absolute 0x03;
volatile unsigned char    _indf     absolute 0x00;
volatile unsigned char    _pcl      absolute 0x02;
volatile unsigned char    _pclath   absolute 0x0A;
volatile unsigned char    _intcon   absolute 0x0B;


#define OS_Init_ProcSpec()      // Empty macro for mikroC


/************************************************************************************************
 *                                                                                              *
 *                           K E R N E L   F U N C T I O N S                                    *
 *                                                                                              *
 ************************************************************************************************/


void __OS_JumpToTask (void)
{
    _OS_SET_FSR_CUR_TASK_W(2);
    _pclath = _indf & 0x1F;
    _fsr--;
    asm {
        movf    __indf, w
        decf    __fsr, f
        movwf   __pcl
    }
}

/*
 ********************************************************************************
 *                                                                              *
 *  void OS_SchedRetPoint (void)                                                *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Return to sheduler with saving current task return point    *
 *                                                                              *
 *  parameters:     char mode:                                                  *
 *                  =0 - only save point of return                              *
 *                  =1 - save retpoint, clear ready, set cancontinue            *
 *                  =2 - save retpoint and clear ready                          *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 ********************************************************************************
 */

void OS_SchedRetPoint (void)
{
    asm{
        clrf    __status
        movwf   r0
    }
    _OS_temp = _fsr;
    _OS_SET_FSR_CUR_TASK_W(2);

    _indf = r0;
    _fsr--;
    _indf = _OS_temp;
    _fsr--;

    _OS_bTaskReady = 1;

    if (r0 & 0x80)
    {
        _OS_bTaskReady = 0;

        #if defined(OS_ENABLE_TTIMER)

            _OS_bTaskCanContinue = 0;
            if (PCLATH & 0x01) _OS_bTaskCanContinue = 1;

        #endif
    }
}


//******************************************************************************
//  
//******************************************************************************

void __OS_SET_FSR_CUR_TASK (void)
{
    _fsr = ((_FSR_TYPE)&_OS_CurTask->State);
    _OS_SET_IRP_CUR_TASK();
}



/*
 ********************************************************************************
 *                                                                              *
 *   void _OS_Task_Create(_OST_INT_TYPE priority, OST_CODE_POINTER TaskAddr)    *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *   description:   (Internal function called by system kernel from service     *
 *                  OS_Task_Create)                                             *
 *                  Create task in free descriptor.                             *
 *                                                                              *
 *  parameters:     priority - value from 0 (highest) to 7 (lowest)             *
 *                  TaskAddr - pointer to C-function that contains task         *
 *                                                                              *
 *  on return:      check OS_IsError                                            *
 *                                                                              *
 ********************************************************************************
 */

//------------------------------------------------------------------------------
#define _OS_Task_Create_DEFINED
//------------------------------------------------------------------------------


void _OS_Task_Create(_OST_INT_TYPE priority, OST_CODE_POINTER TaskAddr)
{
    OST_TASK_POINTER Task;

    OS_Flags.bError = 0;

    #if defined(OS_DISABLE_PRIORITY)
    priority = 0;   // To increase scheduler speed
    #endif



    /*--------------------------------------*
     *                                      *
     *  Start search from first task in     *
     *  OS_TASKS descriptors.               *
     *                                      *
     *--------------------------------------*/
    Task = (OST_TASK_POINTER)OS_TaskVars;
    _OS_temp = OS_TASKS;

    do {
        _OS_SET_IRP_CUR_TASK();

        if (!Task->State.bEnable)               // Is descriptor free?
        {                                       // Yes
            ((OST_TASK_STATE*)&priority)->bEnable = 1;
            ((OST_TASK_STATE*)&priority)->bReady = 1;

            Task->pTaskPointer = TaskAddr;

            #ifdef OS_ENABLE_TTIMERS
                Task->Timer = 0;
            #endif
                            
            #ifdef _OS_TASK_CREATE_PROC_SPEC                                //90700
            _OS_TASK_CREATE_PROC_SPEC;                                      //90700
            #endif                                                          //90700

            *(OS_TASKS_BANK char*)&Task->State = priority;

            #if defined(_OS_CUR_FLAGS_IN_OS_state)                          //90700
            if (Task == _OS_CurTask) ((NEAR char)OS_state) = priority;      //90700
            #endif                                                          //90700

            // Now FSR points to created task descriptor
            // disabled since 90111: _fsr = (_FSR_TYPE)Task;

            OS_Flags.bError = 0;
            _irp = 0;
            return ;

        }

        Task ++;

    } while (--_OS_temp);

    // There is no free descriptor. Task was not created.
    OS_Flags.bError = 1;

    _irp = 0;
    return ;
}



/********************************************************************************
 *
 *  Function:       _OS_Task_GetPriority
 *
 *------------------------------------------------------------------------------
 *
 *  description:    Get priority of current task.
 *                  MikroC does not clear IRP flag after working with pointers to
 *                  bank2/bank3. So we create a function that will set and then
 *                  will clear IRP bit if needed
 *
 *  parameters:     OST_TASK_POINTER pTask - pointer to task
 *
 *  on return:      __osa_uint8 - priority
 *
 ********************************************************************************/

__osa_uint8 _OS_Task_GetPriority (OST_TASK_POINTER pTask)
{
    __osa_uint8 temp;
    _OS_SET_IRP_CUR_TASK();
    temp = pTask->State.cPriority;
    _OS_CLR_IRP();
    return temp;
}



/********************************************************************************
 *
 *  Function:       _OS_Task_GetPriority
 *
 *------------------------------------------------------------------------------
 *
 *  description:    Get state of bPaused bit for task.
 *                  MikroC does not clear IRP flag after working with pointers to
 *                  bank2/bank3. So we create a function that will set and then
 *                  will clear IRP bit if needed
 *
 *  parameters:     OST_TASK_POINTER pTask - pointer to task
 *
 *  on return:      __osa_uint8 - bPaused flag state
 *
 ********************************************************************************/

__osa_uint8 _OS_Task_IsPaused (OST_TASK_POINTER pTask)
{
    __osa_uint8 temp;
    _OS_SET_IRP_CUR_TASK();
    temp = pTask->State.bPaused? 1:0;
    _OS_CLR_IRP();
    return temp;
}









/************************************************************************************************
 *                                                                                              *
 *                            S Y S T E M   F U N C T I O N S                                   *
 *                                                                                              *
 ************************************************************************************************/







/*
 ********************************************************************************
 *                                                                              *
 *  char OS_DI (void)                                                           *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Disable interrupt with GIE saving                           *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      char - bit 7 = previous GIE value, bits 0..6 = 0            *
 *                                                                              *
 ********************************************************************************
 */

//-------------------------------------------------------------------------------
#define OS_DI_DEFINED
//-------------------------------------------------------------------------------

char OS_DI (void)
{
    char temp;
    temp = _intcon & 0x80;
    _intcon &= 0x7F;
    return temp;
}



 
    
    

/*
 ************************************************************************************************
 *                                                                                              *
 *                         C R I T I C A L   S E C T I O N S                                    *
 *                                                                                              *
 ************************************************************************************************
 */
//------------------------------------------------------------------------------
#ifdef OS_ENABLE_CRITICAL_SECTION
//------------------------------------------------------------------------------
    
/*
 ********************************************************************************
 *                                                                              *
 *  void OS_EnterCriticalSection (void)                                         *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Enter critical section.                                     *
 *                  This function disables interrupts (with saving current      *
 *                  state) and sets system flag bInCriticalSection              *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 *  note:           This function overloads "osa.c"                             *
 *                                                                              *
 ********************************************************************************
 */

//------------------------------------------------------------------------------
#define OS_EnterCriticalSection_DEFINED
//------------------------------------------------------------------------------

void OS_EnterCriticalSection (void)
{
    _OST_INT_TYPE temp;
    temp = OS_DI();
    OS_Flags.bInCriticalSection = 1;

    OS_Flags.bGIE_CTemp = 0;
    if (temp & 0x80) OS_Flags.bGIE_CTemp = 1;

}

/*
 ********************************************************************************
 *                                                                              *
 *   void OS_LeaveCriticalSection (void)                                        *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Leave critical section.                                     *
 *                  This function restore interrupt state from OS_Flag temp bits*
 *                  and clears system flag bInCriticalSection                   *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 *  note:           This function overloads "osa.c"                             *
 *                                                                              *
 ********************************************************************************
 */

//------------------------------------------------------------------------------
#define OS_LeaveCriticalSection_DEFINED
//------------------------------------------------------------------------------

void OS_LeaveCriticalSection (void)
{
    char temp;
    OS_Flags.bInCriticalSection = 0;
    temp = 0;
    if (OS_Flags.bGIE_CTemp) temp |= 0x80;
    OS_RI(temp);
}

//------------------------------------------------------------------------------
#endif      //     #ifdef OS_ENABLE_CRITICAL_SECTION
//------------------------------------------------------------------------------


















/*
 ************************************************************************************************
 *                                                                                              *
 *                                   D E L A Y S   I N   T A S K                                *
 *                                                                                              *
 ************************************************************************************************
 */

//------------------------------------------------------------------------------
#ifdef OS_ENABLE_TTIMERS
//------------------------------------------------------------------------------


/*
 ********************************************************************************
 *                                                                              *
 *   void _OS_InitDelay (_OST_TTIMER Delay)                                     *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    (Internal function called by system kernel)                 *
 *                                                                              *
 *                  Init task timer delay.Set timer variable in task descriptor,*
 *                  sets bit bDelay and clears bit bCanContinue.                *
 *                                                                              *
 *                  _OS_CurTask must point to descriptor.                       *
 *                                                                              *
 *                                                                              *
 *  parameters:     Delay   - time of delay in system ticks                     *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 *  note:           This function overloads "osa.c"                             *
 *                                                                              *
 ********************************************************************************
 */

//------------------------------------------------------------------------------
#define _OS_InitDelay_DEFINED
//------------------------------------------------------------------------------


void _OS_InitDelay (OST_TIMER Delay)
    {

        _OS_SET_FSR_CUR_TASK();
        _OS_bTaskCanContinue = 0;
        _OS_bTaskDelay = 0;         // Clear bDelay to prevent timer changing in interrupt

        if (Delay)
        {

            Delay ^= -1;
            Delay ++;
            
            _OS_SET_IRP_CUR_TASK();
            _OS_CurTask->Timer = Delay;
            _OS_SET_FSR_CUR_TASK();
            _OS_bTaskDelay = 1;
            _OS_bTaskReady = 1;

        }
    }

//______________________________________________________________________________
//******************************************************************************

//------------------------------------------------------------------------------
#endif  // #ifdef OS_ENABLE_TTIMERS
//------------------------------------------------------------------------------



    










    
    
    
    
    

    


















    
    
    
    

/*
 ********************************************************************************
 *                                                                              *
 *  bit _OS_CheckEvent (char bEvent)                                            *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    (Internal function called by system kernel througth         *
 *                  waiting services)                                           *
 *                                                                              *
 *                  Check condition of bEvent. Accordint to bEvent value and    *
 *                  current state of bReady, bDelay and bCanContinue flags,     *
 *                  task becames ready to execute or still remains in waiting   *
 *                  mode.                                                       *
 *                                                                              *
 *  parameters:     bEvent - zero, or non-zero condition                        *
 *                                                                              *
 *  on return:      1 - when condition is true, or timeout occured              *
 *                  0 - condition is false and no timeout                       *
 *                                                                              *
 *  note:           This function overloads "osa.c"                             *
 *                                                                              *
 ********************************************************************************
 */

//------------------------------------------------------------------------------
#define _OS_CheckEvent_DEFINED
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
_OST_INT_TYPE _OS_CheckEvent (_OST_INT_TYPE bEvent)
{
    //------------------------------------------------------------------------------
    #ifdef OS_DISABLE_PRIORITY                  // *91103.4*
    //------------------------------------------------------------------------------
    
        OS_Flags.bTimeout = 0;
        _OS_SET_FSR_CUR_TASK();
    
        if (bEvent) 
        {
            #ifdef OS_ENABLE_TTIMERS

                _OS_bTaskDelay = 0; // Clear bDelay in temporary variable OS_state

            #endif
    
            return 1;
    
        }

        #ifdef OS_ENABLE_TTIMERS
    
            if (_OS_bTaskTimeout && _OS_bTaskCanContinue)
            {
                OS_Flags.bTimeout = 1;
                return 1;
            }
    
        #endif
    
        return 0;
        
    //------------------------------------------------------------------------------
    #else
    //------------------------------------------------------------------------------
    
        OS_Flags.bTimeout = 0;
        _OS_SET_FSR_CUR_TASK();
    
        if (bEvent) 
        {
            if (_OS_bTaskReady) 
            {
        EXIT_GOOD:
                #ifdef OS_ENABLE_TTIMERS
    
                    _OS_bTaskDelay = 0; // Clear bDelay in temporary variable OS_state
    
                #endif
    
                return 1;
            }
    
            _OS_bTaskReady = 1;
    
        } else {
            _OS_bTaskReady = 0;
        }
    
        #ifdef OS_ENABLE_TTIMERS
    
            if (_OS_bTaskTimeout && _OS_bTaskCanContinue)
            {
                OS_Flags.bTimeout = 1;
                _OS_bTaskReady = 1;
                goto EXIT_GOOD;
            }
    
        #endif
    
        return 0;

    //------------------------------------------------------------------------------
    #endif
    //------------------------------------------------------------------------------
}


    


        

